package Agent::DSPMESSAGE;
#<copyright>
# ----------------------------------------------------------
# Sun Proprietary/Confidential Code
# Copyright 2001, Sun Microsystems, Inc. All rights reserved.
# ----------------------------------------------------------
#</copyright>
#  $Name:  $ 
#  $Id: DSPMESSAGE.pm,v 1.11 2004/05/06 15:34:59 ccadieux Exp $

use strict;
use State;
use PDM::Parse;
use Report;
use Agent;
use base 'Agent';
use Policies;
use RasThread;

sub isSelectable { 0 }
sub revision     { '$Revision: 1.11 $'}
sub type         { Report::CAT_DSPMESSAGE }
sub diskTotal    { 9    }

use vars qw( $ERR %FOUND);

sub new {
  my($self) = Agent->new();

  bless ($self, 'Agent::DSPMESSAGE');
  return $self;
}
use vars qw ($DONE);


sub RUN {
   my($agent, $static_list) = @_;
   my($err_lines, $warn_lines,  $out,$device, $connect_errs);
   my($report ,  $id, $lines, $log_err, $type);
   my( $wwn, $portWWN, $port2WWN);
   $DB::single = 1;
   my $type = uc($agent->type());
 
   my $processed_list = [];
 
   my $renv     = System->get_renv();
   my $logf     = $renv->{dsp_logfile} || "/var/adm/messages.dsp";
   my $warn_lines = [];
   my $ISP;
   $DONE = 1;

   if ($logf && -r $logf) {
       Debug->print1("Executing agent DSPMESSAGE");
       Timelapse->start("dsp:logfile");
       ($log_err, $lines) = $agent->read_log_file($logf,'NEW');  
       Debug->err('CANNOT_READ', $logf) if ($log_err);
       $agent->find_alert($lines, $device, \$ISP, $warn_lines);
       Timelapse->stop("dsp:logfile");
   }
      
   my($name) = $renv->{hostname} || System->hostname();
   if ($#$warn_lines >= 0) {
      $id = {
            deviceName  => ($renv->{solution} eq "N") ? $renv->{hostname} : System->hostid(),
            active      => "Y",
            logFile     => $logf,
            display     => "$name (id=" . System->hostid() . ")",
            name        => $name, 
            class       => "host",
            category    => Report::CAT_DSPMESSAGE,
            ip          => "",
         };
      $report->{"id.name"} = $name;
      my $new_rep = Report->new($id, $report , $warn_lines, Report::STATUS_DONT_SAVE);
      require Health::Message;
      my $broke_list = Health::Message->all_logic($new_rep);
      foreach my $dev (@$broke_list) {
         $dev->{problems} = 2; # broke but not processed yet.
         push(@$processed_list, $dev);
      }
      PDM->saveReport($new_rep);
   }
 
   return $processed_list;
}



############################################################
#                    SUBROUTINES
############################################################


# 1 = warning
# 2 = errors
# Jul  8 14:45:33 dsp1  07/08/2003 16:45:36 LOG_INFO     (CONFIG: 0-0)  Building config - Device Config complete


sub find_alert {
  my($agent, $lines, $device, $ISP, $err) = @_;
  my( $level, $name, $no1, $no2, $rest);
  my($m, $year, $date, $comp, $x);
  my($ipno);

  my $Config = PDM::ConfigFile->read();
  my $devs = $Config->devices();
  my (%ACTIVE, %IP);
  my $policies = Policies->new("dsp_policies");
  my ($comp1, $comp2, %MAP);

  foreach my $d (@$devs) {
     $IP{$d->{ipno}} = $d->{type} . ":" . $d->{key} if ($d->{ipno});
     $ACTIVE{$d->{ipno}} = $d->{active};
     if ($main::TESTMODE) {
         my $name = $d->{name};
         $MAP{$name} = $d->{ipno};
         my $ix = index($name, "-");
         if ($ix > 0) {
            $name = substr($name, $ix+1);
            $MAP{$name} = $d->{ipno};
         }
     }
  }
  for ($x = 0; $x <= $#$lines; $x++ ) {
     my $line = $lines->[$x];
     my(@a) = split(/\s+/, $line);

     my $t3_ip = $MAP{$a[3]} || Util->name2ip($a[3]);
     $MAP{$a[3]} = $t3_ip if ($a[3] ne $t3_ip);

     my ($key, $knownT3, $key0);
        
     if (exists($IP{$t3_ip})) {
       $FOUND{$t3_ip}++;
       next if ($ACTIVE{$t3_ip} eq "N");

       my $u1 = $a[6]; $knownT3 = 1;

       my $topic = "$a[7] $a[8]";
       $comp1    = "";
       ($comp1, $comp2)  = split(/\:/, substr($topic, 1, -1));

       $key  = $IP{$t3_ip} . ":$comp1";
       $key0 = $IP{$t3_ip};

     } else {
       $key  = "t3:HOST";
       $key0 = "t3:HOST";
     }

     #                            ix  disk    t3a    key    ip     key0
     $policies->run($err, $lines,\$x,$comp1, $a[3], $key, $t3_ip, $key0, $knownT3);
  }         

  return ($err);
}




1;
