package Agent::SVE2;
#<copyright>
# ----------------------------------------------------------
# Sun Proprietary/Confidential Code
# Copyright 2001, Sun Microsystems, Inc. All rights reserved.
# ----------------------------------------------------------
#</copyright>

#  $Name:  $ 
#  $Id: SVE2.pm,v 1.11 2004/04/23 21:42:43 ccadieux Exp $

use strict;
use Agent;
use base 'Agent';
use Net::Telnet;

sub isSelectable {"Sun SVE2"}
sub revision {'$Revision: 1.11 $'}
use vars qw (%WWNS $ERR);

sub category {"sve2"}

sub new {
  my($self) = Agent->new();

  bless ($self, 'Agent::SVE2');
  return $self;
}


sub RUN {
  my($agent, $static_list, $pass) = @_;
  my($HBA, $logfile, $r, @results);
  my($xml, @lux_dif, $dif, $ev, $found);
  my($log_err, $lines, $timelapse);
  my(%DONE, $report, $device, $wwn, $id, $x, $l, $connect_errs);
  $DB::single=1;
  
  my($renv) = System->get_renv();


  my($today) = Util->today("YMDH");
  my $processed_list = [];
  my $dc = 0;

  foreach $device ( $agent->deviceList($static_list)) {
    $dc++;

    Debug->print1("-> " . uc($device->{type}) . ": Reading device $device->{name}/$device->{wwn}");

    ($wwn, $connect_errs, $report) = $agent->INSTRUMENTATION($device);

     if ($connect_errs eq "BUSY") {
       Debug->print2("Device $device->{name} is busy, skipping!");
       next;
     }

     $id = {  
              deviceName  => $device->{wwn},
              active      => $device->{active},
              logFile     => "",
              display     => $device->{name} ,
              name        => $device->{name},
              class       => $device->{class},
              category    => Report::CAT_SVE2,
              ip          => "",
           };
     $report = {} if (!$report);
     $agent->copyDev($device, $report);

     $report->{"id.name"} = $device->{name};
     $report->{"id.ipno"} = System->get_ipno();
     $report->{"id.wwn"} = $device->{wwn};
     my $new_rep;
     if ($connect_errs || !$wwn) {
        $report->{'id.connect_errs'} = $connect_errs;
        $new_rep = Report->new($id, $report , undef, Report::STATUS_CANNOT_CONNECT);

     } else {
        $new_rep = Report->new($id, $report , undef);
     }
     require Health::SVE2;
     my $problems = PDM->get_last_event();
     Health::SVE2->all_logic($new_rep);

     PDM->saveReport($new_rep);

     my ($broke, $abort) = $agent->new_events($problems, $device, 1);
     push(@$processed_list, $device);
     last if ($broke && $pass == 1 || $abort);
  } 
  return $processed_list;
}

############################################################################



sub telnet_login {
  my($agent, $dev, $TO) = @_;

  $TO = 300 if (!$TO);
    my($t) = Net::Telnet->new(
             errmode => "return",
             Timeout   => $TO,
             Prompt    => '/[#?] $/',
             );
    my $ipno = $dev->{ipno};
    my $ix = index($ipno, ":");
    if ($ix > 0) {
      $ipno = substr($ipno,0, $ix);
    }

    if (!defined($t->open($ipno))) {
      return ("Cannot open $ipno", undef);
    }
    my $pass = Util->decode($dev->{telnet});
    $t->login($dev->{login}, $pass);

    if ($t->errmsg()) {
       return ($t->errmsg(), undef);
    }
    return (undef, $t);
}


# dev = {ipno => '1.1.1.1', login => ' ', password => ' '}

sub getWWN {
  my($agent, $dev) = @_;
  my $wwn;
  my ($err, $telnet) = $agent->telnet_login($dev);

  if ($err || !$telnet) {
     Debug->print2("telnet $dev->{name}: $err");
     return $wwn;

  } else {
     my @coms = $telnet->cmd("show chassis eeprom");
     foreach my $l (@coms) {
       if ($l =~ /^MIC\s+Base/) {
          my @L = split(/\s+/, $l);
          $wwn = "$L[2]-$L[3]";
          last;
       }
     }
  }
  $telnet->cmd("exit");
  $telnet->close();
  return $wwn;
}

sub getVersions {
  my($agent, $telnet, $REP) = @_;
  my $wwn;
  my $in;
  my @coms = $telnet->cmd("show version");
  my $cardno=0;
  foreach my $l (@coms) {
     $l = Util->ltrim($l);
     if ($l =~ /====/) {
        $in = 1;
     } elsif ($in) {
          my ($num,$proc,$image,$rev,$date,$time,$user,$host) = split(/\s+/, $l);
          if ($rev) {
            $REP->{"version.$num-$proc.name"}       = $image;
            $REP->{"version.$num-$proc.revision"}   = $rev;
            $REP->{"version.$num-$proc.date"}       = $date;
            $REP->{"version.$num-$proc.time"}       = $time;
            $REP->{"version.$num-$proc.user"}       = $user;
          }
     }
  }
}

sub getCards {
  my($agent, $telnet, $REP) = @_;
  my $wwn;
  my $in;
  my @coms = $telnet->cmd("show chassis eeprom");
  my $cardno=0;
  foreach my $l (@coms) {
     if ($l =~ /====/) {
        $in = 1;
     } elsif ($in) {
          my ($num,$type,$part,$serial,$hw,$pcb,$date,$mac) = split(/\s+/, $l);
          if ($date) {
            $REP->{"card.$num-$type.type"}         = $type;
            $REP->{"card.$num-$type.partNumber"}   = $part;
            $REP->{"card.$num-$type.serialNo"}     = $serial;
            $REP->{"card.$num-$type.hwRevision"}   = $hw;
            $REP->{"card.$num-$type.pcbRevision"}  = $pcb;
            $REP->{"card.$num-$type.mfgDate"}      = $date;
            $REP->{"card.$num-$type.mac"}          = $mac;
          }
     }
  }
}

#                     Disks for sus0 (Chassis ID: 100044)
#     Flags: A   -  Active disk path
#            O/S -  Owner/Secondary array path
#            M   -  Multiple paths to the specified processor
#
#                                                  Free space             Owner 
# Path / Slice    Proc     State       Size        Total/Max       Flags   ID   
#---------------- ---- ------------- --------- ------------------- ----- -------
#disk/3/1/0/0     3-1  Initialized    235.93 GB                     O     Local
#disk/4/1/0/0     4-1  Blocked                                      S  
#              /0      Free          235.93 GB 235.93 GB/235.93 GB

sub getServers {
  my($agent, $telnet, $REP) = @_;
  my $wwn;
  my $in;
  my @coms = $telnet->cmd("show fc-servers");
  my $server = 0;
  foreach my $l (@coms) {
     if ($l =~ /--------/) {
        $in = 1;
     } elsif ($in) {
          my ($name, $ins, $state, $status) = split(/\s+/, Util->ltrim($l));
          if ($status) {
             $REP->{"server.$server.name"}     = $name;
             $REP->{"server.$server.instance"} = $ins;
             $REP->{"server.$server.state"}    = $state;
             $REP->{"server.$server.status"}   = $status;
             $server++;
          }
     }
  }
  $REP->{"info.server_cnt"} = $server+1;
}
  

sub getPower {
  my($agent, $telnet, $REP) = @_;

  my @com = $telnet->cmd("show chassis");
  my($x, $in, $no);
  for ($x=0; $x <= $#com; $x++) {
     my $l = $com[$x];
     if ($com[$x] =~ /Power Supply/) {
        $x++; $in = "power";
     } elsif ($com[$x] =~ /Slot Card/) {
        $x++; $in = "slot";

#Power Supply   State                    Fan Tray State
#-------------+-----------------------+------------------
     } elsif ($in eq "power") {
        $l .= "                                 ";
        my $name  = Util->trim(substr($l,0,8));
        my $state = Util->trim(substr($l,14,10));
        if ($name) {
          $REP->{"power.$name.state"} = $state;
        }

#Slot Card Description                              CPUs Ports State
#----+---------------------------------------------+----+-----+-------------
     } elsif ($in eq "slot") {
        if (length($l) < 50) {
           $x++;
           next;
        }
        $no = Util->trim(substr($l,0,4));
        my $name  = Util->trim(substr($l,5,30));
        my $cpus  = Util->trim(substr($l,51,4));
        my $ports = Util->trim(substr($l,56,5));
        my $state = Util->trim(substr($l,62));
        $REP->{"chassis.$no-1.name"} = $name;
        $REP->{"chassis.$no-1.cpus"} = $cpus;
        $REP->{"chassis.$no-1.ports"} = $ports;
        $REP->{"chassis.$no-1.state"} = $state;
        $x++;
        $l = $com[$x];
        $name  = Util->trim(substr($l,5,30));
        $cpus  = Util->trim(substr($l,51,4));
        $ports = Util->trim(substr($l,56,5));
        $state = Util->trim(substr($l,62));
        $REP->{"chassis.$no-2.name"} = $name;
        $REP->{"chassis.$no-2.cpus"} = $cpus;
        $REP->{"chassis.$no-2.ports"} = $ports;
        $REP->{"chassis.$no-2.state"} = $state;
     }
  }
  $REP->{"info.chassis_cnt"} = $no;
}

#Path/Slice has the SIO card number/port embedded into it.  With the 4 port 
#cards, this is the same as the "Proc" (processor).  As an example, I break down 
#the following:
#disk/3/1/0/0 : disk indicates it is a disk, in this case an array, plugged into 
#SIO board 3, port 1.  In this case, I'm not sure what the first "0" means, but 
#the second "0" is the T3+ LUN number.  It shows another path to this LUN, 
#disk/4/1/0/0, which is plugged into board 4, port 1.  This path is marked as 
#"blocked" since this is the failover path to LUN 0.  The third line is the 
#"slice" number.  For the SAN volumes, there will always be 1 slice, and its 
#number will be "0" as far as I know.  This line tells you how much free space 
#you have on that LUN.
#|                                       Size                     
#| Vendor Name   Product ID         Raw/Initialized      Owner ID
#| -----------   ----------       --------------------  ----------
#|   SUN         T300            116.71 GB/116.59 GB     Local     
#|
#|   Proc        Path          Flags        State                 Port WWN 
#|  ------ ----------------   -------   ---------------   -------------------------
#|   3-3    disk/3/5/7/1        O        Initialized       50:02:0F:23:00:00:06:96
#|   4-3    disk/4/5/6/1        S        Blocked           50:02:0F:23:00:00:06:52
#|

sub getDisks {
  my($agent, $telnet, $REP) = @_;

  my @com = $telnet->cmd("show disks -detailed");

  foreach my $l (@com) {
     $l = Util->ltrim(substr($l,1));
     my($proc, $path, $flag, $state, $wwn) = split(/\s+/, $l);
     if ($wwn =~ /^[\w\:]+$/ && substr($path, 0, 4) eq "disk") {
           $wwn = lc($wwn);
           $wwn =~ s/\://g;
           my @d1 = split(/\//, $path);
           my $key = "$d1[1]/$d1[2]";
           my $port =  $WWNS{"$key.port"};
           $REP->{"port.$port.disk_path"}  = $path;
           $REP->{"port.$port.disk_proc"}  = $proc;
           $REP->{"port.$port.disk_flag"}  = $flag;
           $REP->{"port.$port.disk_state"} = $state;
           $REP->{"port.$port.PortWWN"}    = $wwn;
           $REP->{"port.$port.LocalPortWWN"}   = $WWNS{"$key.WWN"};
     }
  }
  
}

#|-> Port '2 Gig Fibre Channel 3/1  ' (on Storage Processor: 3-1)
#|   (ifIndex: 12)
#|   Type of Port: '2 Gig Fibre Channel', Description: '[Description Not Specified]',
#|   Fibre Channel Configuration:
#|     Port State             : Offline
#|     Hardware State         : Link Down
#|     Port ID                : 00:00:00:00:00:00:00:00
#|     WWN                    : 27:31:00:01:5D:00:7E:00
#|     Node WWN               : 27:00:00:01:5D:00:7E:00
#|     Port Type              : nPort
#|     FC Unit Type           : HBA
#|     Transmitter Type       : Unknown
#|     Module Type            : GBIC
#|     Back Panel Port Number : 1
#|     Up Time                : 8 days 16 hrs 1 mins 55 

sub getPorts {
  my($agent, $telnet, $REP) = @_;

  my @com = $telnet->cmd("show interfaces fibrechannel");

  my ($card, $slot) ;
  my $port=-1;
  foreach my $l (@com) {
      next if ($l !~ /\:/);
      next if (substr($l,0,1) eq "+");
      $l = substr($l,1);
      if ($l =~ /^\-\> Port .* Fibre Channel (\d+\/\d+)/) {
          ($card,$slot) = split(/\//, $1);
          $port++;
          $REP->{"port.$port.card"} = $card;
          $REP->{"port.$port.slot"} = $slot;
      } elsif ($l =~ /(\s*[^\:]+)\s*:\s+(.*)/) {
          my $name = $1; 
          my $val = $2;
          $name =~ s/Gbits\/sec//;
          $name =~ s/[ \(\)]//g;
          if ($name eq "WWN" || $name eq "NodeWWN") {
             $val =~ s/\://g;
             $val = lc($val);
             $WWNS{"$card/$slot.$name"} = $val;
             $WWNS{"$card/$slot.port"} = $port;
          }
          $REP->{"port.$port.$name"} = $val if ($port >= 0);
      }
  }
  $REP->{"info.port_cnt"} = $port+1;
}
 
  
# returns port/wwn information
# (Error, $REP->{...})

sub getLinks {
  my($agent, $device, $report) = @_;

  my $renv  = System->get_renv();
  my $TO    = 60;
  if (!Util->ping($device->{ipno})) {
     return "Cannot ping";
  }
  
  my ($err, $telnet) = $agent->telnet_login($device);

  if ($err || !$telnet) {
     return "BUSY";
  } else {
     $telnet->cmd("no page-mode");
     $telnet->cmd("terminal width 200");
     $agent->getPorts($telnet, $report);
     $agent->getDisks($telnet, $report);
     $telnet->cmd("exit");
     $telnet->close();
  }

  return undef;
}


sub INSTRUMENTATION {
  my($agent, $device) = @_;
  my(@s, %dev, $in, %info, $key, $state, $num);
  my($v3, $v1, $v2, $x, $wwn, $status, $rev, $ib, $no, $id);

  my($renv)  = System->get_renv();
  my $TO     = 60;
  if (!Util->ping($device->{ipno})) {
     return ($device->{key}, "Cannot ping", undef);
  }
  %WWNS = ();
  my ($err, $telnet) = $agent->telnet_login($device);

  if ($err || !$telnet) {
     Debug->print2("telnet $device->{name}: $err");
     return ($device->{key}, "BUSY", undef);

  } else {
     Timelapse->startDev( $device);
     $telnet->cmd("no page-mode");
     $telnet->cmd("terminal width 200");
     $agent->getPorts($telnet, \%info);
     $agent->getVersions($telnet, \%info);
     $agent->getDisks($telnet, \%info);
     $agent->getCards($telnet, \%info);
     $agent->getServers($telnet, \%info);
     $agent->getPower($telnet, \%info);
     $agent->addIdentification(\%info);
     $telnet->cmd("exit");
     $telnet->close();
     Timelapse->stopDev( $device);
  }

  return ($device->{key}, "", \%info);
}





sub REPORT {
  my($class, $host, $r, $arg) = @_;

  my($out);
  my($tableCnt) = $arg->{tableCnt};
  my($rev) = $r->{'_value'}{"info.FirmwareVersion"};
  my($host0) = $host || System->hostname();
  $out .= $class->reportHead('SVE2', $r);
  my $rep = $r->{'_value'};
  $out .= "
   <tr><td bgcolor=#F0F0F0 align=right><b>Name:</td><td width=25%>&nbsp;$r->{_id}{name}</td>
   <td bgcolor=#F0F0F0 align=right><b>IP:</td><td width=25%>&nbsp;$rep->{'id.device_ip'}</td>
  
   <tr><td bgcolor=#F0F0F0 align=right><b>Key:</td><td colspan=3>&nbsp;$r->{_id}{deviceName}</td>
   <tr><td bgcolor=#F0F0F0 align=right><b>Power:</td><td colspan=3>&nbsp;A=$rep->{'power.A.state'} / B=$rep->{'power.B.state'}</td>

   <tr><td bgcolor=#F0F0F0 align=right><b>Ports:</td><td>&nbsp;$rep->{'info.port_cnt'}</td>
       <td bgcolor=#F0F0F0 align=right colspan=2>&nbsp;</td>
  </table>";

######################################
# CHASSIS

  $out .= "
  <table border=1 cellspacing=0 cellpadding=1 width=95% bgcolor=white>
  <tr bgcolor=$Style::DARK><td colspan=5><font color=white><b>Chassis</td>
  <tr bgcolor=$Style::LIGHT>
      <th>#
      <th>Name
      <th>Cpus
      <th>Ports
      <th>Status
   ";
  my($x);
  for ($x=0; $x < $rep->{'info.chassis_cnt'}; $x++) {
    if ($rep->{"chassis.$x-1.name"}) {
      $out .= "\n<tr><td>$x-1".
              "<td>" . $rep->{"chassis.$x-1.name"} .
              "<td><center>" . $rep->{"chassis.$x-1.cpus"} .
              "<td><center>" . $rep->{"chassis.$x-1.ports"} .
              "<td>" . $rep->{"chassis.$x-1.state"};
    }
    if ($rep->{"chassis.$x-2.name"}) {
      $out .= "\n<tr><td>$x-2".
              "<td>" . $rep->{"chassis.$x-2.name"} .
              "<td><center>" . $rep->{"chassis.$x-2.cpus"} .
              "<td><center>" . $rep->{"chassis.$x-2.ports"} .
              "<td>" . $rep->{"chassis.$x-2.state"} ;
    }
  }
  $out .= "</table>";

######################################
# SERVERS

  $out .= "
  <table border=1 cellspacing=0 cellpadding=1 width=95% bgcolor=white>
  <tr bgcolor=$Style::DARK><td colspan=5><font color=white><b>Servers</td>
  <tr bgcolor=$Style::LIGHT>
      <th>Name
      <th>Instance
      <th>State
      <th>Status
   ";
  my($x);
  for ($x=0; $x < $rep->{"info.server_cnt"}; $x++) {
       $out .= "\n<tr>".
              "<td>" . $rep->{"server.$x.name"}     . "</td>".
              "<td>" . $rep->{"server.$x.instance"} . "</td>" .
              "<td>" . $rep->{"server.$x.state"}    . "</td>" .
              "<td>" . $rep->{"server.$x.status"} ;
  }
  $out .= "</table>";

######################################
# CARDS

  $out .= "
  <table border=1 cellspacing=0 cellpadding=1 width=95% bgcolor=white>
  <tr bgcolor=$Style::DARK><td colspan=5><font color=white><b>Cards</td>
  <tr bgcolor=$Style::LIGHT><th>CardId
      <th>Type
      <th>Part#
      <th>Serial#
      <th>Rev.
   ";
  my($x);
  foreach my $el (sort keys %$rep) {
     if ($el =~ /card\.(.*)\.type/) {
       my $key = $1;
       $out .= "\n<tr><td>$key</td>".
              "<td>" . $rep->{"card.$key.type"} .
              "<td>" . $rep->{"card.$key.partNumber"} .
              "<td>" . $rep->{"card.$key.serialNo"} .
              "<td>" . $rep->{"card.$key.hwRevision"} . "/ ". 
                  $rep->{"card.$key.pcbRevision"} ;
     }
  }
  $out .= "</table>";

######################################
# VERSIONS

  $out .= "
  <table border=1 cellspacing=0 cellpadding=1 width=95% bgcolor=white>
  <tr bgcolor=$Style::DARK><td colspan=5><font color=white><b>Versions</td>
  <tr bgcolor=$Style::LIGHT><th>CardId
      <th>Revision
      <th>Date
      <th>Time
   ";
  my($x);
  foreach my $el (sort keys %$rep) {
     if ($el =~ /version\.(.*)\.revision/) {
       my $key = $1;
       $out .= "\n<tr><td>$key</td>" .
              "<td>" . $rep->{"version.$key.revision"} .
              "<td>" . $rep->{"version.$key.date"} .
              "<td>" . $rep->{"version.$key.time"} 
                ;
     }
  }
  $out .= "</table>";

######################################
# PORTS

  $out .= "
  <table border=1 cellspacing=0 cellpadding=1 width=95% bgcolor=white>
  <tr bgcolor=$Style::DARK><td colspan=5><font color=white><b>PortWWN</td>
  <tr bgcolor=$Style::LIGHT><th>#<th>Type
      <th>WWN
      <th>Module
      <th>State
   ";
  for ($x=0; $x < $rep->{'info.port_cnt'}; $x++) {
      my $no = $rep->{"port.$x.card"} . "-" . $rep->{"port.$x.slot"};
      $out .= "\n<tr><td>$no<td>" . $rep->{"port.$x.ProtocolOperational"} .
              "<td>" . $rep->{"port.$x.WWN"} .
              "<td>" . $rep->{"port.$x.ModuleType"} .
              "<td>" . $rep->{"port.$x.PortState"}
              ;
  }
  $out .= "</table>";
    

  $out .= "
  <table border=1 cellspacing=0 cellpadding=1 width=95% bgcolor=white>
  <tr bgcolor=$Style::DARK><td colspan=2><font color=white><b>Ports</td>
   ";
  my($x);
  for ($x=0; $x < $rep->{'info.port_cnt'}; $x++) {
    my $card = $rep->{"port.$x.card"};
    my $port = $rep->{"port.$x.slot"};
    $out .= "<tr><td colspan=2 bgcolor=$Style::LIGHT><b>Card=$card, Port=$port</td>";
    foreach my $el (sort keys %$rep) {
      if ($el =~ /^port.$x.(.*)/ ) {
        my $name = $1; 
        if (index(",FCClassCapable,FCUnitType,HardwareState,ModuleType,NodeWWN,NumberofPorts,PortState,PortType,Speed,UpTime,", ",$name,") >= 0) {
          my $val = $rep->{$el};
          $out .= "<tr><td align=right bgcolor=#F0F0F0>$name: </td><td>$val&nbsp;</td>";
        }
      }
    }
  }
  $out .= "</table>";
  if (!$tableCnt) {
    $out .= "</table>\n";
  }
  return $out;

}




1;

